import sdl2 

from aerforgesdl.vec2 import Vec2
from aerforgesdl.error import ForgeError

class Input:
    def __init__(self):
        self._key = sdl2.SDL_GetKeyboardState(None)
        self._mouse_pos = Vec2(0, 0)

        self._key_pressed = False

        self._pressed_key_name = ""

        self._mouse_pressed = False

        self._mouse_left_pressed = False
        self._mouse_middle_pressed = False
        self._mouse_right_pressed = False

        self._mouse_motion = False

    def update(self, event):
        self._key_pressed = False

        self._pressed_key_name = ""

        self._mouse_pressed = False

        self._mouse_left_pressed = False
        self._mouse_middle_pressed = False
        self._mouse_right_pressed = False

        self._mouse_motion = False

        if event.type == sdl2.SDL_MOUSEMOTION:
            self._mouse_pos.x, self._mouse_pos.y = event.motion.x, event.motion.y

        if event.type == sdl2.SDL_KEYDOWN:
            self._key_pressed = True
            self._pressed_key_name = sdl2.SDL_GetKeyName(event.key.keysym.sym).decode()

        if event.type == sdl2.SDL_MOUSEBUTTONDOWN:
            if event.button.button == sdl2.SDL_BUTTON_LEFT:
                self._mouse_left_pressed = True

            if event.button.button == sdl2.SDL_BUTTON_MIDDLE:
                self._mouse_middle_pressed = True

            if event.button.button == sdl2.SDL_BUTTON_RIGHT:
                self._mouse_right_pressed = True

            self._mouse_pressed = True
            self._mouse_motion = True


    def key_pressed(self, key = None):
        if key == None:
            return self._key_pressed

        else:
            return self._key[key]

    def pressed_key_name(self):
        return self._pressed_key_name

    def mouse_pressed(self, button = None):
        if button == None:
            return self._mouse_pressed

        elif button == sdl2.SDL_BUTTON_LEFT:
            return self._mouse_left_pressed

        elif button == sdl2.SDL_BUTTON_MIDDLE:
            return self._mouse_middle_pressed

        elif button == sdl2.SDL_BUTTON_RIGHT:
            return self._mouse_right_pressed

        else:
            raise ForgeError("Invalid button")

    def mouse_pos(self):
        return self._mouse_pos